type instruction =
  | Nop
  | Print of IMPExpr.expression
  | Exit
  | Label    of string
  | Jump     of IMPExpr.expression (* adresse *)
  | JumpWhen of IMPExpr.expression (* adresse *)
              * IMPExpr.expression (* condition *)
  | Write of IMPExpr.expression (* adresse *)
           * IMPExpr.expression (* valeur *)
      
type program = { text: instruction list; data: (string * int) list }

let rec instr_to_string = function
  | Print(e) -> "print(" ^ (IMPExpr.to_string e) ^ ");"
  | Exit -> "exit;"
  | _ -> failwith "not implemented (ART.instr_to_string)"
    
let rec seq_to_string = function
  | [] -> ""
  | i::s -> (instr_to_string i) ^ "\n" ^ (seq_to_string s)

let rec data_to_string = function
  | [] -> ""
  | _ -> failwith "not implemented (ART.data_to_string)"
    
let to_string p =
  ".text\n" ^ (seq_to_string p.text) ^ "\n.data\n" ^ (data_to_string p.data)
    
